/* wmthermistor - WindowMaker Temperature Monitor DockApp
 * state.c
 * reads temperatures and set some values
 * Copyright (C) 2002 Fabio Pugliese Ornellas <fabio.ornellas@poli.usp.br>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <unistd.h>

// temperatures
float temp[3][58];
// max / min
float max[2];
float min[2];
// counters
extern int i;
extern int j;

/* this function reads on the /proc filesystem the temperatures, from the module
  w83782d; you may change the lines below to use it with you sensor
  module. please send it to me!! (see AUTHOR)*/

// read temperatures
void get_temp ( int discart[4] ) {
  // current values
  float max_last=0.0;
  float min_last=0.0;

  FILE *fp1;
  FILE *fp2;
  FILE *fp3;

  // first check if hardware is supported
  if( !fopen("/proc/sys/dev/sensors/w83782d-isa-0290/temp1","r") ) {
    fprintf(stderr,"Error!\nUnable to open file /proc/sys/dev/sensors/w83782d-isa-0290/temp1 for reading!\nUnsupported hardware driver or module w83781d not present!\nSee README for more info!\n");
     _exit(1);
  }

  // temperature stack down
  for ( i=0 ; i <= 2 ; i++ )
    for ( j=57 ; j >= 1 ; j-- )
      temp[i][j] = temp[i][j-1];

  // sensor 1
  fp1 = fopen ( "/proc/sys/dev/sensors/w83782d-isa-0290/temp1", "r" );
  fscanf ( fp1, "%*s %*s %f", &temp[0][0] );
  fclose(fp1);

  // sensor 2
  fp2 = fopen ( "/proc/sys/dev/sensors/w83782d-isa-0290/temp2", "r" );
  fscanf ( fp2, "%*s %*s %f", &temp[1][0] );
  fclose(fp2);

  // sensor 3
  fp3 = fopen ( "/proc/sys/dev/sensors/w83782d-isa-0290/temp3", "r" );
  fscanf ( fp3, "%*s %*s %f", &temp[2][0] );
  fclose(fp3);

  // discart thermistors
  switch(discart[0]) {
  // one thermistor out
  case 1:
    temp[2][0]=(temp[0][0]+temp[1][0])/2;
    break;
  case 2:
    temp[1][0]=(temp[0][0]+temp[2][0])/2;
    break;
  case 4:
    temp[0][0]=(temp[1][0]+temp[2][0])/2;
    break;
  // two thermistors out
  case 3:
    temp[1][0]=temp[0][0]+5;
    temp[2][0]=temp[0][0]-5;
    break;
  case 5:
    temp[0][0]=temp[1][0]+5;
    temp[2][0]=temp[1][0]-5;
    break;
  case 6:
    temp[0][0]=temp[2][0]+5;
    temp[1][0]=temp[2][0]-5;
    break;
  }


  // sets the graph max height
  // if current is max
  if ( temp[0][0] >= temp[1][0] && temp[0][0] >= temp[2][0] )
    max_last = temp[0][0];
  if ( temp[1][0] >= temp[0][0] && temp[1][0] >= temp[2][0] )
    max_last = temp[1][0];
  if ( temp[2][0] >= temp[1][0] && temp[2][0] >= temp[0][0] )
    max_last = temp[2][0];
  if ( max_last >= max[1] ) {
    max[0] = 0.0;
    max[1] = max_last;
  }
  // case max is out...
  if ( max[0] > 57.0 ) {
    max[1] = -1000.0;
    // ...look for a new max
    for ( i=0 ; i<=2 ; i++ )
      for ( j=0 ; j<=57 ; j++ ) {
        if ( temp[i][j] >= max[1] ) {
          max[0] = (float)j;
          max[1] = temp[i][j];
        }
      }
  }
  // sets the graph min height
  // similar as above
  if ( temp[0][0] <= temp[1][0] && temp[0][0] <= temp[2][0] )
    min_last = temp[0][0];
  if ( temp[1][0] <= temp[0][0] && temp[1][0] <= temp[2][0] )
    min_last = temp[1][0];
  if ( temp[2][0] <= temp[1][0] && temp[2][0] <= temp[0][0] )
    min_last = temp[2][0];
  if ( min_last <= min[1] ) {
    min[0] = 0.0;
    min[1] = min_last;
  }
  if ( min[0] > 57.0 ) {
    min[1] = 1000.0;
    for ( i=0 ; i<=2 ; i++ )
      for ( j=0 ; j<=57 ; j++ ) {
        if ( temp[i][j] <= min[1] ) {
          min[0] = (float)j;
          min[1] = temp[i][j];
        }
      }
  }
}
