/* wmthermistor - WindowMaker Temperature Monitor DockApp
 * wmthermistor.c
 * Copyright (C) 2002 Fabio Pugliese Ornellas <fabio.ornellas@poli.usp.br>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <unistd.h>
#include <dockapp.h>
#include "wmthermistor.xpm"

char *displayName = "";
// default delay, equals to module update delay
int delay = 1400;
// scale factor for the bars
float scale;
// discart thermistor
// chmod like notation:
// blue=4, yellow=2, green=1
// add these values to give whitch ones to discart
// example: 5 would discart thermistors green and blue
int discart[4];

// temperatures
extern float temp[3][58];
// max / min
extern float max[2];
extern float min[2];
// coutners
int i;
int j;

// get_temp
void get_temp ( int discart[4] );

GC gc;
Pixmap pixmap;
Pixmap backdrop;
// draw asked number
// (number, temperature sensor, x position)
void number ( int n, int t, int x ) {
  XCopyArea ( DADisplay, backdrop, pixmap, gc, n*4, 120+( t*7 ), 4, 7, x, 1 );
}

// draw temperature numbers
// (temperature sensor, value)
void value ( int t, float v ) {
  // clear display
  XCopyArea ( DADisplay, backdrop, pixmap, gc, 1 + (20*t), 61, 14, 7, 1 + (20*t) , 1 );
  if ( v >= 100.0 ) {
  // 3 digit number
    // x--
    number ( (int)(v/100.0), t, 1 + (20*t) );
    // -x-
    number ( ((int)(v/10.0))%10 , t, 6+(20*t) );
    // --x
    number ( (int)(v) % ((int)(v/10.0)*10), t,11+(20*t) );
  }
  else {
    if ( v >= 10.0 ) {
    // 2 digit number
      // x-
      number ( ((int)(v/10.0)) , t, 6+(20*t) );
      // -x
      number ( (int)(v)%10, t,11+(20*t) );
    } else {
    // 1 digit number
      // x
      number ( (int)(v), t, 11+(20*t) );
    }
  }
  // .5 / .0 degree value
  if ( (int)(v*10.0) % (int)(v) == 0 ) {
  // integer value
    XCopyArea ( DADisplay, backdrop, pixmap, gc, 44, 120+( t*7 ), 3, 7, 16 + (20*t) , 1 );
  } else {
  // five
    XCopyArea ( DADisplay, backdrop, pixmap, gc, 47, 120+( t*7 ), 3, 7, 16 + (20*t) , 1 );
  }
}

// command line
static DAProgramOption options[] = {
  {"-d", "--display", "display to use", DOString, False, {&displayName} },
  {"-i", "--interval", "interval between updates (in miliseconds)", DONatural, False, {&delay} },
  {"-f", "--full", "draw full bars instead of points", DONone, False, {NULL}},
  {"-c", "--discart", "discart thermistor (chmod like:\n                               blue=4, yellow=2, green=1\n                               add the values of the one(s) to disable)", DOInteger, False, {&discart} }
};

// draw bars
// (temperature sensor, position)
void bar ( int t, int i ) {
  int bar_size;
  // do not draw previous values
  if ( temp[t][i] != 1000.0) {
    // var size
    bar_size = (int) ( ( (temp[t][i] - min[1]) * scale ) + 3.0);
    if (options[2].used)
      // full bar
      XCopyArea ( DADisplay, backdrop, pixmap, gc, 60+t, 0, 1, bar_size, 58-i, 59-bar_size );
    else
      // dots only
      XCopyArea ( DADisplay, backdrop, pixmap, gc, 60+t, 1, 1, 1, 58-i, 59-bar_size );
  }
}

int main (int argc, char **argv) {
  static unsigned b_width = 63;
  static unsigned b_height = 141;

  // define callbacks
  DACallbacks callbacks = { NULL, NULL, NULL, NULL, NULL, NULL };

  // initial values to max/ min temperatures
  // [0] = position
  // [1] = value
  max[0] = 0.0;
  min[0] = 0.0;
  max[1] = -1000.0;
  min[1] = 1000.0;
  // discart = none
  for (i=0;i<4;i++)
    discart[i]=1;

  // "zero" previous temperatures not readed (zero=1000)
  for ( i=0 ; i <= 2 ; i++ )
    for ( j=0 ; j <= 57 ; j++ )
      temp[i][j] = 1000.0;

  // Parse CommandLine
  DAParseArguments(argc, argv, options, sizeof(options) / sizeof(DAProgramOption), "WindowMaker Temperature Monitor DockApp", "wmthermistor 0.2");

  // welcome message
  printf ( "wmthermistor - WindowMaker Temperature Monitor Dock App\nVersion 0.2\nTHIS SOFTWARE COMES WITH ABSOLUTELY NO WARRANTY! USE AT YOUR OWN RISK!\n\n");

  // invalid delay is fixed
  if ( delay == 0 ) {
    fprintf ( stderr, "Warning: invalid interval 0,  reseting to 1400 ms\n" );
    delay = 1400;
  }

  // pharse discart thermistor
  if(discart[0]>7) {
    fprintf(stderr,"Invalid value to discarting thermistors. Valid values are natural lower than 8.\n");
    return 1;
  }
  // green
  if(discart[0]%2!=0)
    discart[3]=0;
  // yellow
  if(discart[0]>1&&discart[0]!=4&&discart[0]!=5)
    discart[2]=0;
  // blue
  if(discart[0]>3)
    discart[1]=0;

  // convert microseconds to miliseconds
  delay = delay * 1000 ;

  // Initialize Application
  DAInitialize(displayName, "wmthermistor", 60, 60, argc, argv);
  DAMakePixmapFromData(wmthermistor_xpm, &backdrop, None, &b_width, &b_height);
  pixmap = DAMakePixmap();
  gc = DefaultGC(DADisplay, DefaultScreen(DADisplay));
  DASetCallbacks (&callbacks);
  DAShow();

  // splash screen
  XCopyArea ( DADisplay, backdrop, pixmap, gc, 0, 0, 60, 60, 0, 0);
  DASetPixmap(pixmap);

  usleep (1000000);

  // draw background
  XCopyArea ( DADisplay, backdrop, pixmap, gc, 0, 60, 60, 60, 0, 0);
  DASetPixmap(pixmap);

  // update loop
  for(;;) {
    // starts xevents
    XEvent ev;

    // clear graph
    XCopyArea ( DADisplay, backdrop, pixmap, gc, 1, 69, 58, 50, 1, 9 );

    // get current temperarures
    get_temp(discart);

    // calculate scale factor
    scale = (46.0 / (max[1] - min[1]));

    // draw bars in the correct order ( bigger to smaller ) to avoid draw over last one
    for ( i=0 ; i<=57 ; i++ ) {
      if ( temp[0][i] >= temp[1][i] ) {
        if ( temp[0][i] >= temp[2][i] ) {
          if ( temp[1][i] >= temp[2][i] ) {
          //  0 1 2
            if(discart[1])
              bar(0,i);
            if(discart[2])
              bar(1,i);
            if(discart[3])
              bar(2,i);
          } else {
          // 0 2 1
            if(discart[1])
              bar(0,i);
            if(discart[3])
              bar(2,i);
            if(discart[2])
              bar(1,i);
          }
        } else {
        // 2 0 1
          if(discart[3])
            bar(2,i);
          if(discart[1])
            bar(0,i);
          if(discart[2])
            bar(1,i);
        }
      } else {
        if ( temp[1][i] >= temp[2][i] ) {
          if ( temp[0][i] >= temp[2][i] ) {
          // 1 0 2
            if(discart[2])
              bar(1,i);
            if(discart[1])
              bar(0,i);
            if(discart[3])
              bar(2,i);
          } else {
          // 1 2 0
            if(discart[2])
              bar(1,i);
            if(discart[3])
              bar(2,i);
            if(discart[1])
              bar(0,i);
          }
        } else {
        // 2 1 0
          if(discart[3])
            bar(2,i);
          if(discart[2])
            bar(1,i);
          if(discart[1])
            bar(0,i);
        }
      }
    }
    // fix position of max/ min
    max[0]++;
    min[0]++;

    // draw temperatures
    if(discart[1])
      value ( 0, temp[0][0] );
    if(discart[2])
      value ( 1, temp[1][0] );
    if(discart[3])
      value ( 2, temp[2][0] );

    // do pending xevents
    while ( XPending ( DADisplay ) ) {
      XNextEvent(DADisplay, &ev);
      DAProcessEvent(&ev);
    }

    // apply updates
    DASetPixmap(pixmap);

    // run delay
    usleep(delay);
  }

  // this should never be seen...
  fprintf(stderr, "Congratulations, something truely awful has happened.\n");
  return -1;
}
